function kMeansDemo2(dataset)

% ------------------------ parameters section -------------------------------
if nargin<1
        dataset = './data/blobs01.mat';
end

interactive = 1;

% ------------------------ here be the code ---------------------------------
% load dataset - also try with other .mat files (e.g. circles02.mat, ...)
load(dataset);

fprintf('I have loaded dataset %s\n',dataset);
fprintf('Dataset characteristics: %d points, %d-dimensional space, %d clusters\n\n',size(X,1),size(X,2),clusters);
% plot data
fprintf('Plotting data...\n');
figure;plotClustering(X,classes,classes);drawnow();
title('Data plot (colors/shapes represent different classes)')
fprintf('\nIn the previous demo, you have seen that k-means does not always\n');
fprintf('return the same clustering results. This is due to the fact that\n');
fprintf('centroids are randomly initialized, causing results to be very different.\n\n');
fprintf('A way to obtain better results is to run k-means many times,\n');
fprintf('measure its quality, and keep only the best results.\n\n');
fprintf('How can we understand, however, whether a result is good or bad?\n\n');
fprintf('We will use INTERNAL indices, that allow us to evaluate clusters\n');
fprintf('without requiring external information.\n\n');
fprintf('So, how does it work? We will run k-means many times (e.g. 10),\n');
fprintf('calculating SSE every time and keeping only the results providing\n');
fprintf('us the smallest SSE.\n\n');
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

bestSSE = 1e6; % huge number as a worst-case SSE
bestLabels = [];
% run kmeans N times (feel free to vary N)
maxiter = 10;
for i = 1 : maxiter
        [centroid, pointsInCluster, labels]= myKmeans(X, clusters);
        sse = SSE(X,centroid,labels);
        fprintf('Execution %02d: SSE=%.6f\n',i,sse);
        if sse < bestSSE
                bestSSE = sse;
                bestLabels = labels;
        end
end

fprintf('Keeping clustering results which gave us SSE=%.6f\n',bestSSE);
figure;plotClustering(X,bestLabels,classes);
title(sprintf('Results of k-means (k=%d) providing the lowest SSE out of 10 tests. Accuracy: %.4f',clusters,accuracy(bestLabels,classes)));
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

fprintf('Good! Feel free to play with the source code (i.e. changing the number\n');
fprintf('of tests) or with other datasets. From the next demo, SSE code will be\n');
fprintf('moved to the function repKmeans(X, clusters, iterations).\n\n');
end
