function [acc]=uAveragedAccuracy(labels,classes,debug)
% micro averaged accuracy (also often called "clustering accuracy"
% provides a measure of accuracy not just for a single cluster,
% but for a whole clustering at once. 
% Actually, rather than an accuracy this is a /precision/ measure.
% It can be considered as an accuracy measure when the number of
% different labels equals the number of classes (i.e. when the
% number of guessed clusters is equal to the number of actual ones)

if (nargin < 3)
  debug = 0;
end

totsize = 0;
totmax = 0;

labs = unique(labels);
for i = 1:length(labs)
    labelsize = length(find(labels==labs(i)));
    subclust = unique(classes(labels==labs(i)));
    maxsize = 0;
    for j = 1:length(subclust)
        clustsize = length(find(classes(labels==labs(i))==subclust(j)));
        if clustsize>maxsize
           maxsize = clustsize;
        end
    end 
    labelprec = maxsize / labelsize;
    if debug
        fprintf('Label %d => maxsize = %d, labelsize = %d, precision = %.4f\n', labs(i), maxsize, labelsize, labelprec);
    end
    totsize = totsize + labelsize;
    totmax = totmax + maxsize;
end
    acc = totmax/totsize;
    if debug
        fprintf('Totmax = %d, Totsize = %d, accuracy = %.4f\n', totmax, totsize, acc);
    end
end
